package de.intevation.mxd.reader;

import java.awt.Color;

import org.apache.log4j.Logger;

import com.esri.arcgis.display.ISymbol;
import com.esri.arcgis.display.ILineSymbol;
import com.esri.arcgis.display.CartographicLineSymbol;
import com.esri.arcgis.display.IRgbColor;
import com.esri.arcgis.display.RgbColor;
import com.esri.arcgis.display.Template;

import com.esri.arcgis.display.esriLineCapStyle;
import com.esri.arcgis.display.esriLineJoinStyle;

import org.w3c.dom.Element;
import java.io.IOException;

/**
 * Reads cartoline symbol information.
 *
 * @author <a href="mailto:raimund.renkert@intevation.de">Raimund Renkert</a>
 */
public class CartoLineSymbolReader
extends AbstractSymbolReader {

    /**
     * The logger.
     */
    private static final Logger logger =
        Logger.getLogger(CartoLineSymbolReader.class);

    /**
     * Private member.
     */
    private CartographicLineSymbol symbol;

    public CartoLineSymbolReader(ISymbol symbol)
    throws Exception {
        logger.debug("contructor()");
        if(symbol instanceof CartographicLineSymbol) {
            this.symbol = (CartographicLineSymbol)symbol;
        }
        else {
            throw new Exception("Not a CartographicLineSymbol!");
        }
    }


    public CartoLineSymbolReader(ILineSymbol symbol)
    throws Exception {
        logger.debug("contructor()");
        if(symbol instanceof CartographicLineSymbol) {
            this.symbol = (CartographicLineSymbol)symbol;
        }
        else {
            throw new Exception("Not a CartographicLineSymbol!");
        }
    }

    /**
     * Reads the symbol attributes.
     *
     * @return The XML node.
     */
    public Element read() {
        logger.debug("read()");
        Element symbolElement = util.addSymbol(parent);

        try {
            symbolElement.setAttribute("name", symbol.getNameString());
        }
        catch(IOException ioe) {
            logger.warn("Could not read name. Setting name to \"default\"");
            symbolElement.setAttribute("name", "default");
        }

        try {
            if(symbol.getColor() instanceof IRgbColor) {
                IRgbColor color = (IRgbColor)symbol.getColor();
                Color c = new Color (
                    color.getRed(),
                    color.getGreen(),
                    color.getBlue());
                symbolElement.setAttribute("color", String.valueOf(c.getRGB()));
            }
            else {
                RgbColor col = new RgbColor();
                col.setRGB(symbol.getColor().getRGB());
                Color c = new Color (
                    col.getRed(),
                    col.getGreen(),
                    col.getBlue());
                symbolElement.setAttribute("color", String.valueOf(c.getRGB()));
            }
        }
        catch(IOException ioe) {
            logger.warn("Could not read color.");
        }

        int cap;
        String capStyle = "";
        try {
            cap = symbol.getCap();
        }
        catch(IOException ioe) {
            logger.warn(
                "Could not read line cap.");
            cap = -1;
        }
        switch(cap) {
            case esriLineCapStyle.esriLCSRound: capStyle = "round"; break;
            case esriLineCapStyle.esriLCSSquare: capStyle = "square"; break;
            default: break;
        }


        int join;
        String joinStyle = "";
        try {
            join = symbol.getJoin();
        }
        catch(IOException ioe) {
            logger.warn(
                "Could not read line join." +
                " Setting line join to \"bevel\".");
            join = -1;
        }
        switch(join) {
            case esriLineJoinStyle.esriLJSRound: joinStyle = "round"; break;
            case esriLineJoinStyle.esriLJSMitre: joinStyle = "miter"; break;
            default: break;
        }
        if(!capStyle.equals("")) {
            symbolElement.setAttribute("cap", capStyle);
        }
        if (!joinStyle.equals("")) {
            symbolElement.setAttribute("join", joinStyle);
        }

        try {
            symbolElement.setAttribute(
                "linestart",
                String.valueOf(symbol.getLineStartOffset()));
        }
        catch(IOException ioe) {
            logger.warn("Could not read line start offset.");
        }

        try {
            symbolElement.setAttribute(
                "miterlimit",
                String.valueOf(symbol.getMiterLimit()));
        }
        catch(IOException ioe) {
            logger.warn("Could not read miter limit.");
        }

        try {
            symbolElement.setAttribute(
                "offset",
                String.valueOf(symbol.getOffset()));
        }
        catch(IOException ioe) {
            logger.warn("Could not read offset.");
        }

        try {
            symbolElement.setAttribute(
                "width",
                String.valueOf(symbol.getWidth()));
        }
        catch(IOException ioe) {
            logger.warn("Could not read width.");
        }
        //TODO Read further attributes depending on the Mapscript functionality:
        //     Template, MarkerSymbol, LineDecorations

        try {
            Template temp = (Template)symbol.getTemplate();
            double length = temp.getInterval();
            int count = temp.getPatternElementCount();
            symbolElement.setAttribute("pattern_count", String.valueOf(count));
            for (int j = 0; j < count; j++) {
                double[] mark = new double[1];
                double[] gap = new double[1];
                temp.getPatternElement(j, mark, gap);
                symbolElement.setAttribute(
                    "mark_" + j,
                    String.valueOf(mark[0]));
                symbolElement.setAttribute("gap_" + j, String.valueOf(gap[0]));
            }
        }
        catch(IOException ioe){
            logger.warn("Could not read pattern elements.");
        }
        symbolElement.setAttribute("type", "line");
        symbolElement.setAttribute("style", "carto");

        return symbolElement;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
